"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = getUser;
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const grab_host_names_1 = __importDefault(require("../../utils/grab-host-names"));
const api_get_user_1 = __importDefault(require("../../functions/api/users/api-get-user"));
/**
 * # Get User
 */
function getUser(_a) {
    return __awaiter(this, arguments, void 0, function* ({ key, userId, database, fields, apiUserId, }) {
        /**
         * Initialize
         */
        const defaultFields = [
            "id",
            "first_name",
            "last_name",
            "email",
            "username",
            "image",
            "image_thumbnail",
            "verification_status",
            "date_created",
            "date_created_code",
            "date_created_timestamp",
            "date_updated",
            "date_updated_code",
            "date_updated_timestamp",
        ];
        const updatedFields = fields && fields[0] ? [...defaultFields, ...fields] : defaultFields;
        const reqPayload = JSON.stringify({
            userId,
            database,
            fields: [...new Set(updatedFields)],
        });
        const grabedHostNames = (0, grab_host_names_1.default)();
        const { host, port, scheme } = grabedHostNames;
        /**
         * Check for local DB settings
         *
         * @description Look for local db settings in `.env` file and by pass the http request if available
         */
        const { DSQL_DB_HOST, DSQL_DB_USERNAME, DSQL_DB_PASSWORD, DSQL_DB_NAME } = process.env;
        if ((DSQL_DB_HOST === null || DSQL_DB_HOST === void 0 ? void 0 : DSQL_DB_HOST.match(/./)) &&
            (DSQL_DB_USERNAME === null || DSQL_DB_USERNAME === void 0 ? void 0 : DSQL_DB_USERNAME.match(/./)) &&
            (DSQL_DB_PASSWORD === null || DSQL_DB_PASSWORD === void 0 ? void 0 : DSQL_DB_PASSWORD.match(/./)) &&
            (DSQL_DB_NAME === null || DSQL_DB_NAME === void 0 ? void 0 : DSQL_DB_NAME.match(/./)) &&
            global.DSQL_USE_LOCAL) {
            /** @type {import("../../types").DSQL_DatabaseSchemaType | undefined} */
            let dbSchema;
            try {
                const localDbSchemaPath = path_1.default.resolve(process.cwd(), "dsql.schema.json");
                dbSchema = JSON.parse(fs_1.default.readFileSync(localDbSchemaPath, "utf8"));
            }
            catch (error) { }
            return yield (0, api_get_user_1.default)({
                userId,
                fields: [...new Set(updatedFields)],
                dbFullName: DSQL_DB_NAME,
            });
        }
        /**
         * Make https request
         *
         * @description make a request to datasquirel.com
         */
        const httpResponse = yield new Promise((resolve, reject) => {
            const httpsRequest = scheme.request({
                method: "POST",
                headers: {
                    "Content-Type": "application/json",
                    "Content-Length": Buffer.from(reqPayload).length,
                    Authorization: key ||
                        process.env.DSQL_FULL_ACCESS_API_KEY ||
                        process.env.DSQL_API_KEY,
                },
                port,
                hostname: host,
                path: `/api/user/${apiUserId || grabedHostNames.user_id}/get-user`,
            }, 
            /**
             * Callback Function
             *
             * @description https request callback
             */
            (response) => {
                var str = "";
                response.on("data", function (chunk) {
                    str += chunk;
                });
                response.on("end", function () {
                    resolve(JSON.parse(str));
                });
                response.on("error", (err) => {
                    reject(err);
                });
            });
            httpsRequest.write(reqPayload);
            httpsRequest.end();
        });
        /** ********************************************** */
        /** ********************************************** */
        /** ********************************************** */
        return httpResponse;
    });
}
