"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = loginSocialUser;
const addAdminUserOnLogin_1 = __importDefault(require("../../backend/addAdminUserOnLogin"));
const varDatabaseDbHandler_1 = __importDefault(require("../../backend/varDatabaseDbHandler"));
/**
 * Function to login social user
 * ==============================================================================
 * @description This function logs in the user after 'handleSocialDb' function finishes
 * the user creation or confirmation process
 */
function loginSocialUser(_a) {
    return __awaiter(this, arguments, void 0, function* ({ user, social_platform, invitation, database, additionalFields, debug, }) {
        const finalDbName = database ? database : "datasquirel";
        const dbAppend = database ? `\`${finalDbName}\`.` : "";
        const foundUserQuery = `SELECT * FROM ${dbAppend}\`users\` WHERE email=? AND social_id=? AND social_platform=?`;
        const foundUserValues = [user.email, user.social_id, social_platform];
        const foundUser = yield (0, varDatabaseDbHandler_1.default)({
            database: finalDbName,
            queryString: foundUserQuery,
            queryValuesArray: foundUserValues,
            debug,
        });
        if (!(foundUser === null || foundUser === void 0 ? void 0 : foundUser[0]))
            return {
                success: false,
                payload: null,
            };
        let csrfKey = Math.random().toString(36).substring(2) +
            "-" +
            Math.random().toString(36).substring(2);
        let userPayload = {
            id: foundUser[0].id,
            first_name: foundUser[0].first_name,
            last_name: foundUser[0].last_name,
            username: foundUser[0].username,
            user_type: foundUser[0].user_type,
            email: foundUser[0].email,
            social_id: foundUser[0].social_id,
            image: foundUser[0].image,
            image_thumbnail: foundUser[0].image_thumbnail,
            verification_status: foundUser[0].verification_status,
            social_login: foundUser[0].social_login,
            social_platform: foundUser[0].social_platform,
            csrf_k: csrfKey,
            logged_in_status: true,
            date: Date.now(),
        };
        if (additionalFields === null || additionalFields === void 0 ? void 0 : additionalFields[0]) {
            additionalFields.forEach((key) => {
                userPayload[key] = foundUser[0][key];
            });
        }
        if (invitation && (!database || (database === null || database === void 0 ? void 0 : database.match(/^datasquirel$/)))) {
            (0, addAdminUserOnLogin_1.default)({
                query: invitation,
                user: userPayload,
            });
        }
        let result = {
            success: true,
            payload: userPayload,
            csrf: csrfKey,
        };
        return result;
    });
}
